/* parsequerytest.cc: Tests of omqueryparser
 *
 * ----START-LICENCE----
 * Copyright 1999,2000,2001 BrightStation PLC
 * Copyright 2001,2002 Ananova Ltd
 * Copyright 2002,2003 Olly Betts
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 * -----END-LICENCE-----
 */

#include <config.h>
#include "queryparser.h"
#include <iostream>
#include <string>

using namespace std;

Xapian::Query::op default_op = Xapian::Query::OP_OR;

typedef struct {
    const char *query;
    const char *expect;
} test;

static test tests[] = {
    { "om-example", "(om:(pos=1) PHRASE 2 exampl:(pos=2))" },
    { "size_t", "(size:(pos=1) PHRASE 2 t:(pos=2))" },
    { "muscat -wine", "(muscat:(pos=1) AND_NOT wine:(pos=2))" },
    { "a- grade", "(a-:(pos=1) OR grade:(pos=2))" },
    { "gtk+ -gimp", "(gtk+:(pos=1) AND_NOT gimp:(pos=2))" },
    { "c++ -c--", "(c++:(pos=1) AND_NOT c--:(pos=2))" },
    { "Mg2+ Cl-", "(Rmg2+:(pos=1) OR Rcl-:(pos=2))" },
    { "\"c++ standard\"", "(c++:(pos=1) PHRASE 2 standard:(pos=2))" },
    { "AT&T M&S", "(Rat&t:(pos=1) OR Rm&s:(pos=2))" },
    { "E.T. N.A.T.O AB.C.", "(Ret:(pos=1) OR Rnato:(pos=2) OR (Rab:(pos=3) PHRASE 2 c:(pos=4)))" },
    { "author:orwell animal farm", "(Aorwel:(pos=1) OR anim:(pos=2) OR farm:(pos=3))" },
    { "Call to undefined function: imagecreate()", "(Rcall:(pos=1) OR to:(pos=2) OR undefin:(pos=3) OR function:(pos=4) OR imagecr:(pos=5))" },
    { "mysql_fetch_row(): supplied argument is not a valid MySQL result resource", "((mysql:(pos=1) PHRASE 3 fetch:(pos=2) PHRASE 3 row:(pos=3)) OR suppli:(pos=4) OR argument:(pos=5) OR is:(pos=6) OR not:(pos=7) OR a:(pos=8) OR valid:(pos=9) OR Rmysql:(pos=10) OR result:(pos=11) OR resourc:(pos=12))" },
    { "php date() nedelands", "(php:(pos=1) OR date:(pos=2) OR nedeland:(pos=3))" },
    { "hhle", "hoehl:(pos=1)" },
    // Not the obvious parse, but not unreasonable
    { "DVD+RW", "(Rrw:(pos=2) AND_MAYBE Rdvd:(pos=1))" },
    { "wget domein --http-user", "(wget:(pos=1) OR domein:(pos=2) OR (http:(pos=3) PHRASE 2 user:(pos=4)))" },
    { "@home problemen", "(home:(pos=1) OR problemen:(pos=2))" },
    { "'ipacsum'", "ipacsum:(pos=1)" },
    { "canal + ", "canal:(pos=1)" },
    { "/var/run/mysqld/mysqld.sock", "(var:(pos=1) PHRASE 5 run:(pos=2) PHRASE 5 mysqld:(pos=3) PHRASE 5 mysqld:(pos=4) PHRASE 5 sock:(pos=5))" },
    { "\"QSI-161 drivers\"", "(Rqsi:(pos=1) PHRASE 3 R161:(pos=2) PHRASE 3 driver:(pos=3))" },
    { "\"e-cube\" barebone", "((e:(pos=1) PHRASE 2 cube:(pos=2)) OR barebon:(pos=3))" },
    { "\"./httpd: symbol not found: dlopen\"", "(httpd:(pos=1) PHRASE 5 symbol:(pos=2) PHRASE 5 not:(pos=3) PHRASE 5 found:(pos=4) PHRASE 5 dlopen:(pos=5))" },
    { "ERROR 2003: Can't connect to MySQL server on 'localhost' (10061)",
      "(Rerror:(pos=1) OR R2003:(pos=2) OR (Rcan:(pos=3) PHRASE 2 t:(pos=4)) OR connect:(pos=5) OR to:(pos=6) OR Rmysql:(pos=7) OR server:(pos=8) OR on:(pos=9) OR localhost:(pos=10) OR R10061:(pos=11))" },
    { "location.href = \"\"", "(locat:(pos=1) PHRASE 2 href:(pos=2))" },
    { "method=\"post\" action=\"\">", "(method:(pos=1) OR post:(pos=2) OR action:(pos=3))" },
    { "NOT windows", "Syntax: <expression> NOT <expression>" },
    { "hard\xa0space", "(hard:(pos=1) OR space:(pos=2))" },
    { " white\r\nspace\ttest ", "(white:(pos=1) OR space:(pos=2) OR test:(pos=3))" },
    { "!!!MAGIC!!!", "(weird:(pos=1) OR stuff:(pos=2))" }, // !!!MAGIC!!! replaced below
    { "one AND/two/three", "(one:(pos=1) AND (two:(pos=2) PHRASE 2 three:(pos=3)))" },
    { "one +/two/three", "((two:(pos=2) PHRASE 2 three:(pos=3)) AND_MAYBE one:(pos=1))" },
    { "one//two", "(one:(pos=1) PHRASE 2 two:(pos=2))" },
    // These are currently parse errors, but many shouldn't be:
    { "behuizing 19\" inch", NULL },
    { "\"missing quote", NULL }, //"(miss:(pos=1) PHRASE 2 quot:(pos=2))" },
    { "\"phrase one \"phrase two\"", NULL },
    { "19\" rack", NULL },
    { "3,5\" mainboard", NULL },
    { "NEAR 207 46 249 27", NULL },
    { "- NEAR 12V voeding", NULL },
    { "553 sorry, that domain isn't in my list of allowed rcpthosts (#5.7.1)", NULL },
    { "data error (clic redundancy check)", NULL },
    { "? mediaplayer 9\"", NULL },
    { "IRQL_NOT_LESS_OR_EQUAL", NULL },
    { "date(\"w\")", NULL },
    { "\"ctrl+v+c+a fout\"", NULL },
    { "\"xterm -fn\"", NULL },
    { "Syntaxisfout (operator ontbreekt ASP", NULL },
    { "Request.ServerVariables(\"logon_user\")", NULL },
    { "ip_masq_new(proto=TCP)", NULL },
    { "ASP \"request.form\" van \\\"enctype=\"MULTIPART/FORM-DATA\"\\\"", NULL },
    { "\"document.write(\"", NULL },
    { "USER ftp (Login failed): Invalid shell: /sbin/nologin", NULL },
    { "ERROR 1045: Access denied for user: 'root@localhost' (Using password: NO)", NULL },
    { "TIP !! subtitles op TV-out (via DVD max g400)", NULL },
    { "Gigabyte 8PE667 (de Ultra versie) of Asus A7N8X Deluxe", NULL },
    { "\"1) Ze testen 8x AF op de GFFX tegen \"", NULL },
    { "\") Ze houden geen rekening met de kwaliteit van AF. Als ze dat gedaan hadden dan waren ze tot de conclusie gekomen dat Performance AF (dus Bilinear AF) op de 9700Pro goed te vergelijken is met Balanced AF op de GFFX. En dan hadden ze ook gezien dat de GFFX niet kan tippen aan de Quality AF van de 9700Pro.\"", NULL },
    { "\"Ze houden geen rekening met de kwaliteit van AF. Als ze dat gedaan hadden dan waren ze tot de conclusie gekomen dat Performance AF (dus Bilinear AF) op de 9700Pro goed te vergelijken is met Balanced AF op de GFFX. En dan hadden ze ook gezien dat de GFFX niet kan tippen aan de Quality AF van de 9700Pro.\"", NULL },
    { "$structure = imap_header($mbox, $tt);", NULL },
    { "\"ifup: Could not get a valid interface name: -> skipped\"", NULL },
    { "Er kan geen combinatie van filters worden gevonden om de gegevensstroom te genereren. (Error=80040218)", NULL },
    { "ereg_replace(\"\\\\\",\"\\/\"", NULL },
    { "\\\\\"divx+geen+geluid\\\\\"", NULL },
    { "lcase(\"string\")", NULL },
    { "isEmpty( )  functie in visual basic", NULL },
    { "+windows98 +(geen communicatie) +ie5", NULL },
    { "*** stop: 0x0000001E (0xC0000005,0x00000000,0x00000000,0x00000000)", NULL },
    { "Server.CreateObject(\"ADODB.connection\")", NULL },
    { "Presario 6277EA-XP model P4/28 GHz-120GB-DVD-CDRW (512MBWXP) (470048-012)", NULL },
    { "delphi CreateOleObject(\"MSXML2.DomDocument\")", NULL },
    { "Unhandled exeption in IEXPLORE.EXE (FTAPP.DLL)", NULL },
    { "IBM High Rate Wireless LAN PCI Adapter (Low Profile Enabled)", NULL },
    { "asp ' en \"", NULL },
    { "Hercules 3D Prophet 8500 LE 64MB (OEM, Radeon 8500 LE)", NULL },
    { "session_set_cookie_params(echo \"hoi\")", NULL },
    { "windows update werkt niet (windows se", NULL },
    { "De statuscode van de fout is ( 0 x 4 , 0 , 0 , 0 )", NULL },
    { "sony +(u20 u-20)", NULL },
    { "Active Desktop (Hier issie)", NULL },
    { "Asus A7V8X (LAN + Sound)", NULL },
    { "Novell This pentium class machine (or greater) lacks some required CPU feature(s", NULL },
    { "sql server install fails error code (-1)", NULL },
    { "waarom \"~\" in directorynaam", NULL },
    { "Cannot find imap library (libc-client.a).", NULL },
    { "session_register(\"login\");", NULL },
    { "\"kylix+ndmb\"", NULL },
    { "cd'r NEAR toebehoren", NULL },
    { "If ($_SESSION[\"Login\"] == 1)", NULL },
    { "You have an error in your SQL syntax near '1')' at line 1", NULL },
    { "ASRock K7VT2 (incl. LAN)", NULL },
    { "access query \"NOT IN\"", NULL },
    { "[crit] (17)File exists: unable to create scoreboard (name-based shared memory failure)", NULL },
    { "directories lokaal php (uitlezen OR inladen)", NULL },
    { "(multi pc modem)+ (line sync)", NULL },
    { "xp 5.1.2600.0 (xpclient.010817-1148)", NULL },
    { "DirectDraw test results: Failure at step 5 (User verification of rectangles): HRESULT = 0x00000000 (error code) Direct3D 7 test results: Failure at step 32 (User verification of Direct3D rendering): HRESULT = 0x00000000 (error code) Direct3D 8 test results: Failure at step 32 (User verification of Direct3D rendering): HRESULT = 0x00000000 (error code) Direct3D 9 test results: Failure at step 32 (User verification of Direct3D rendering): HRESULT = 0x00000000 (error code)", NULL },
    { "Thermaltake Aquarius II waterkoeling (kompleet voor P4 en XP)", NULL },
    { "E3501 unable to add job to database (EC=-2005)", NULL },
    { "\"arp -s\" ip veranderen", NULL },
    { "header(\"content-type: application/octet-stream\");", NULL },
    { "$datum = date(\"d-m-Y\");", NULL },
    { "\"'\" +asp", NULL },
    { "+session +[", NULL },
    { "Dit apparaat kan niet starten. (Code 10)", NULL },
    { "\"You cannot use the Administration program while the Domino Server is running. Either shut down the Domino Server (but keep the file server running) or choose the ican labeled 'Lotus Notes' instead.\"", NULL },
    { "\"+irq +veranderen +xp\"", NULL },
    { "\"is not a member of 'operator``global namespace''' + c++", NULL },
    { "mkdir() failed (File exists) php", NULL },
    { "laatsteIndex(int n)", NULL },
    { "\"line+in\" OR \"c8783\"", NULL },
    { "if ($_POST['Submit'])", NULL },
    { "NEC DVD+-RW ND-1300A", NULL },
    { "*String not found* (*String not found*.)", NULL },
    { "MSI G4Ti4200-TD 128MB (GeForce4 Ti4200)", NULL },
    { "href=\"#\"", NULL },
    { "Request.ServerVariables(\"REMOTE_USER\") javascript", NULL },
    { "XF86Config(-4) waar", NULL },
    { "Unknown (tag 2000)", NULL },
    { "KT4V(MS-6712)", NULL },
    { "scheduled+AND+nieuwsgroepen+AND+updaten", NULL },
    { "137(netbios-ns)", NULL },
    { "HARWARE ERROR, TRACKING SERVO (4:0X09:0X01)", NULL },
    { "Chr(10) wat is code van \" teken", NULL },
    { "wat is code van \" teken", NULL },
    { "The Jet VBA file (VBAJET.dll for 16-bit version, VBAJET32.dll version", NULL },
    { "Permission denied (publickey,password,keyboard-interactive).", NULL },
    { "De lees- of schrijfbewerking (\"written\") op het geheugen is mislukt", NULL },
    { "Primary IDE channel no 80 conductor cable installed\"", NULL },
    { "\"beeld+loopt+door\"", NULL },
    { "\"2020 NEAR zoom\"", NULL },
    { "setcookie(\"naam\",\"$user\");", NULL },
    { "MSI 645 Ultra (MS-6547) Ver1", NULL },
    { "if ($HTTP", NULL },
    { "data error(cyclic redundancy check)", NULL },
    { "UObject::StaticAllocateObject <- (NULL None) <- UObject::StaticConstructObject <- InitEngine", NULL },
    { "Failure at step 8 (Creating 3D Device)", NULL },
    { "Call Shell(\"notepad.exe\",", NULL },
    { "2.5\\\"harddisk converter", NULL },
    { "creative labs \"dvd+rw\"", NULL },
    { "\"het beleid van deze computer staat u niet toe interactief", NULL },
    { "ati radeon \"driver cleaner", NULL },
    { "\"../\" path", NULL },
    { "(novell client) workstation only", NULL },
    { "Unable to find libgd.(a|so) anywhere", NULL },
    { "\"libstdc++-libc6.1-1.so.2\"", NULL },
    { "ipsec_setup (/etc/ipsec.conf, line 1) cannot open configuration file \"/etc/ipsec.conf\" -- `' aborted", NULL },
    { "Forwarden van domeinnaam (naar HTTP adres)", NULL },
    { "Compaq HP, 146.8 GB (MPN-286716-B22) Hard Drives", NULL },
    { "httpd (no pid file) not running", NULL },
    { "apache httpd (pid file) not running", NULL },
    { "Klasse is niet geregistreerd  (Fout=80040154).", NULL },
    { "\"dvd+r\" \"dvd-r\"", NULL },
    { "\"=\" tekens uit csvfile", NULL },
    { "libc.so.6(GLIBC_2.3)", NULL },
    { "Sitecom Broadband xDSL / Cable Router 4S (DC-202)", NULL },
    { "(t-mobile) bereik", NULL },
    { "error LNK2001: unresolved external symbol \"public", NULL },
    { "patch linux exploit -p)", NULL },
    { "MYD not found (Errcode: 2)", NULL },
    { "ob_start(\"ob_gzhandler\"); file download", NULL },
    { "ECS Elitegroup K7VZA (VIA VT8363/VT8363A)", NULL },
    { "ASUS A7V8X (LAN + Serial-ATA + Firewire + Raid + Audio)", NULL },
    { "Javascript:history.go(-1)", NULL },
    { "java :) als icon", NULL },
    { "onmouseover=setPointer(this", NULL },
    { "\" in vbscript", NULL },
    { "IRC (FAQ OR (hulp NEAR bij))", NULL },
    { "setProperty(\"McSquare\"+i, _xscale, _xscale++);", NULL },
    { "[warn] Apache does not support line-end comments. Consider using quotes around argument: \"#-1\"", NULL },
    { "(php.ini) (memory_limit)", NULL },
    { "line 8: syntax error near unexpected token `kernel_thread(f'", NULL },
    { "VXD NAVEX()@)", NULL },
    { "\"Iiyama AS4314UT 17\" \"", NULL },
    { "include (\"$id.html\");", NULL },
    { "include id.Today's date is: <? print (date (\"M d, Y\")); ?>hp", NULL },
    { "(program files\\common) opstarten", NULL },
    { "\"versterker+bruggen+na+filter\"", NULL },
    { "java \" string", NULL },
    { "+=", NULL },
    { "php +=", NULL },
    { "[php] ereg_replace(\".\"", NULL },
    { "\"echo -e\" kleur", NULL },
    { "adobe premiere \"-1\"", NULL },
    { "DVD brander \"+\" en \"-\"", NULL },
    { "inspirion \"dvd+R\"", NULL },
    { "asp 0x80040E14)", NULL },
    { "\"e-tech motorola router", NULL },
    { "bluetooth '1.3.2.19\"", NULL },
    { "ms +-connect", NULL },
    { "php+print+\"", NULL },
    { "athlon 1400 :welke videokaart\"", NULL },
    { "+-dvd", NULL },
    { "glftpd \"-new-\"", NULL },
    { "\"scandisk + dos5.0", NULL },
    { "socket\\(\\)", NULL },
    { "msn (e-tech) router", NULL },
    { "Het grote Epox 8k3a+ ervaring/prob topic\"", NULL },
    { "\"CF+bluetooth\"", NULL },
    { "kwaliteit (s-video) composite verschil tv out", NULL },
    { "Wie kan deze oude hardware nog gebruiken\" Deel", NULL },
    { "Public Declare Sub Sleep Lib \"kernel32\" (ByVal dwMilliseconds As Long)", NULL },
    { "for inclusion (include_path='.:/usr/share/php')", NULL },
    { "\"muziek 2x zo snel\"\"", NULL },
    { "execCommand('inserthorizontalrule'", NULL },
    { "specs: IBM PS/2, Intel 8086 @ 25 mhz!!, 2 mb intern, 50 mb hd, 5.5\" floppy drive, toetsenbord en geen muis", NULL },
    { "History: GetEventTool <- GetMusicManager <- GetMusicScript <- DMCallRoutine <- AMusicScriptEvent::execCallRoutine <- UObject::execClassContext <- (U2GameInfo M08A1.U2GameInfo0 @ Function U2.U2GameInfo.NotifyLevelChangeEnd : 0075 line 744) <- UObject::ProcessEvent <- (U2GameInfo M08A1.U2GameInfo0, Function U2.U2GameInfo.NotifyLevelChangeEnd) <- UGameEngine::LoadMap <- LocalMapURL <- UGameEngine::Browse <- ServerTravel <- UGameEngine::Tick <- UpdateWorld <- MainLoop", NULL },
    { "Support AMD XP 2400+ & 2600+ (K7T Turbo2 only)", NULL },
    { "'\"><br>bla</br>", NULL },
    { "The instruction at \"0x30053409\" referenced memory at \"0x06460504\". The memory could not be \"read'. Click OK to terminate the application.", NULL },
    { "\"(P5A-b)\"", NULL },
    { "(13,5 > 13) == no-go!", NULL },
    { "eth not found \"ifconfig -a\"", NULL },
    { "<META NAME=\"ROBOTS", NULL },
    { "lp0: using parport0 (interrupt-driven)", NULL },
    { "ULTRA PC-TUNING, COOLING & MODDING (4,6)", NULL },
    { "512MB PC2700 DDR SDRAM Rood (Dane-Elec)", NULL },
    { "header(\"Content Type: text/html\");", NULL },
    { "\"-RW\" \"+RW\"", NULL },
    { "\\\"cresta digital answering machine", NULL },
    { "Arctic Super Silent PRO TC (Athlon/P3 - 2,3 GHz)", NULL },
    { "c++ fopen \"r+t\"", NULL },
    { "c++ fopen (r+t)", NULL },
    { "\"DVD+R\"", NULL },
    { "Class.forName(\"jdbc.odbc.JdbcOdbcDriver\");", NULL },
    { "perl(find.pl)", NULL },
    { "\"-5v\" voeding", NULL },
    { "\"-5v\" power supply", NULL },
    { "An Error occurred whie attempting to initialize the Borland Database Engine (error $2108)", NULL },
    { "(error $2108) Borland", NULL },
    { "On Friday 04 April 2003 09:32, Edwin van Eersel wrote: > ik voel me eigenlijk wel behoorlijk kut :)", NULL },
    { "Elektrotechniek + \"hoe bevalt het?\"\"", NULL },
    { "Shortcuts in menu (java", NULL },
    { "detonator+settings\"", NULL },
    { "(ez-bios) convert", NULL },
    { "Sparkle 7100M4 64MB (GeForce4 MX440)", NULL },
    { "freebsd \"boek OR newbie\"", NULL },
    { "for (;;) c++", NULL },
    { "1700+-2100+", NULL },
    { "PHP Warning:  Invalid library (maybe not a PHP library) 'libmysqlclient.so'", NULL },
    { "NEC DV-5800B (Bul", NULL },
    { "org.jdom.input.SAXBuilder.<init>(SAXBuilder.java)", NULL },
    { "AMD Athlon XP 2500+ (1,83GHz, 512KB)", NULL },
    { "'q ben\"", NULL },
    { "getsmbfilepwent: malformed password entry (uid not number)", NULL },
    { "ude onderdelen\"", NULL },
    { "Heeft iemand enig idee waarom de pioneer (zelf met originele firmware van pioneer) bij mij niet wil flashen ??", NULL },
    { "asus a7v266 bios nieuw -(a7v266-e)", NULL },
    { "cybercom \"dvd+r\"", NULL },
    { "AMD PCNET Family Ethernet Adapter (PCI-ISA)", NULL },
    { "relais +/-", NULL },
    { "formules (slepen OR doortrekken) excel", NULL },
    { "\"%English", NULL },
    { "select max( mysql", NULL },
    { "leejow(saait", NULL },
    { "'Windows 2000 Advanced Server\" netwerkverbinding valt steeds weg", NULL },
    { "K7T Turbo 2  (MS-6330)", NULL },
    { "failed to receive data from the client agent. (ec=1)", NULL },
    { "\"cannot find -lz\"", NULL },
    { "undefined reference to `mysql_drop_db'\"", NULL },
    { "search form asp \"%'", NULL },
    { "(dvd+r) kwaliteit", NULL },
    { "Fatal error: Allowed memory size of 8388608 bytes exhausted (tried to allocate 35 bytes)", NULL },
    { "geluid (schokt OR hapert)", NULL },
    { "Het wordt pas echt leuk als het hard staat!! >:)", NULL },
    { "Uw configuratie bestand bevat instellingen (root zonder wachtwoord) die betrekking hebben tot de standaard MySQL account. Uw MySQL server draait met deze standaard waardes, en is open voor ongewilde toegang, het wordt dus aangeraden dit op te lossen", NULL },
    { "(library qt-mt) not found", NULL },
    { "Qt (>= Qt 3.0.3) (library qt-mt) not found", NULL },
    { "setup was unable to find (or could not read) the language specific setup resource dll, unable to continue. Please reboot and try again.", NULL },
    { "Titan TTC-D5TB(4/CU35)", NULL },
    { "[php] date( min", NULL },
    { "EPOX EP-8RDA+ (nForce2 SPP+MCP-T) Rev. 1.1", NULL },
    { "554 5.4.6 Too many hops 53 (25 max)", NULL },
    { "ik had toch nog een vraagje: er zijn nu eigenlijk alleen maar schijfjes van 4.7GB alleen straks zullen er vast schijfjes van meer dan 4.7GB komen. Zal deze brander dit wel kunnen schijven?" "?(na bijvoorbeeld een firmware update?) ben erg benieuwd", NULL }, // Split ? and ? to avoid trigram problems
    { "ati linux drivers (4.3.0)", NULL },
    { "ENCAPSED_AND_WHITESPACE", NULL },
    { "lpadmin: add-printer (set device) failed: client-error-not-possible", NULL },
    { "welke dvd \"+r\" media", NULL },
    { "Warning: stat failed for fotos(errno=2 - No such file or directory)", NULL },
    { "dvd +/-", NULL },
    { "7vaxp +voltage mod\"", NULL },
    { "lpt port (SPP/EPP) is enabled", NULL },
    { "getenv(\"HTTP_REFERER\")", NULL },
    { "Error setting display mode: CreateDevice failed (D3DERR_DRIVERINTERNALERROR)", NULL },
    { "Exception number: c0000005 (access violation)", NULL },
    { "header(\"Content-type:application/octetstream\");", NULL },
    { "java.security.AccessControlException: access denied (java.lang.RuntimePermission accessClassInPackage.sun.jdbc.odbc)", NULL },
    { "(001.part.met", NULL },
    { "Warning: mail(): Use the -f option (5th param) to include valid reply-to address ! in /usr/home/vdb/www/mail.php on line 79", NULL },
    { "PHP Use the -f option (5th param)", NULL },
    { "dvd \"+\" \"-\"", NULL },
    { "bericht  ( %)", NULL },
    { "createobject(\"Outlook.Application\")", NULL },
    { "2500+ of 2600+ (niett OC)", NULL },
    { "maxtor windows xp werkt The drivers for this device are not installed. (Code 28)", NULL },
    { "Warning: stat failed for /mnt/web/react/got/react/board/non-www/headlines/tnet-headlines.txt (errno=2 - No such file or directory) in /mnt/web/react/got/react/global/non-www/templates/got/functions.inc.php on line 303", NULL },
    { "apm: BIOS version 1.2 Flags 0x03 (Driver version 1.16)", NULL },
    { "GA-8IHXP(3.0)", NULL },
    { "8IHXP(3.0)", NULL },
    { "nasi (de ~ (m.))", NULL },
    { "header(\"Content-Disposition: attachment;", NULL },
    { "\"header(\"Content-Disposition: attachment;\"", NULL },
    { "\"Beep -f\"", NULL },
    { "kraan NEAR (Elektrisch OR Electrisch)", NULL },
    { "checking for Qt... configure: error: Qt (>= Qt 3.0.2) (headers and libraries) not found. Please check your installation!", NULL },
    { "parse error, unexpected '\\\"', expecting T_STRING or T_VARIABLE or T_NUM_STRING", NULL },
    { "ac3 (0x2000) \"Dolby Laboratories,", NULL },
    { "Movie.FileName=(\"../../../~animations/\"+lesson1.recordset.fields('column3')+\"Intro.avi\")", NULL },
    { "502 Permission Denied - Permission Denied - news.chello.nl -- http://www.chello.nl/ (Typhoon v1.2.3)", NULL },
    { "Motion JPEG (MJPEG codec)", NULL },
    { ": zoomtext\"", NULL },
    { "Your SORT command does not seem to support the \"-r -n -k 7\"", NULL },
    { "Geef de naam van de MSDOS prompt op C:\\\\WINDOWS.COM\\\"", NULL },
    { "\"\"wa is fase\"", NULL },
    { "<v:imagedata src=\"", NULL },
    { "system(play ringin.wav); ?>", NULL },
    { "\"perfect NEAR systems\"", NULL },
    { "LoadLibrary(\"mainta/gamex86.dll\") failed", NULL },
    { "DATE_FORMAT('1997-10-04 22:23:00', '%W %M %Y');", NULL },
    { "Warning: stat failed for /mnt/web/react/got/react/board/non-www/headlines/tnet-headlines.txt (errno=2 - Onbekend bestand of map) in /mnt/web/react/got/react/global/non-www/templates/got/functions.inc.php on line 303", NULL },
    { "secundaire IDE-controller (dubbele fifo)", NULL },
    { "\"Postal2+Explorer.exe\"", NULL },
    { "COUNT(*)", NULL },
    { "Nuttige Windows progs   (1/11)", NULL },
    { "if(usercode==passcode==)", NULL },
    { "lg 8160b (dvd+r)", NULL },
    { "iPAQ Pocket PC 2002 End User Update (EUU - Service Pack)", NULL },
    { "'ipod pakt tags niet\"", NULL },
    { "ac3 (0x2000) \"Dolby Laboratories", NULL },
    { "\"DVD+/-R\"", NULL },
    { "\"DVD+R DVD-R\"", NULL },
    { "php ;)  in een array zetten", NULL },
    { "De inhoud van uw advertentie is niet geschikt voor plaatsing op marktplaats! (001", NULL },
    { "creative (soundblaster OR sb) 128", NULL },
    { "Can't open file: (errno: 145)", NULL },
    { "Formateren lukt niet(98,XP)", NULL },
    { "access denied (java.io.", NULL },
    { "(access denied (java.io.)", NULL },
    { "wil niet installeren ( crc fouten)", NULL },
    { "(DVD+RW) brandsoftware meerdere", NULL },
    { "(database OF databases) EN geheugen", NULL },
    { "(server 2003) winroute", NULL },
    { "54MHz (kanaal 2 VHF) tot tenminste 806 MHz (kanaal 69 UHF)", NULL },
    { "(draadloos OR wireless) netwerk", NULL },
    { "localtime(time(NULL));", NULL },
    { "ob_start(\"ob_gzhandler\");", NULL },
    { "PPP Closed : LCP Time-out (VPN-0)", NULL },
    { "COM+-gebeurtenissysteem", NULL },
    { "rcpthosts (#5.7.1)", NULL },
    { "Dit apparaat werkt niet goed omdat Windows de voor dit apparaat vereiste stuurprogramma's niet kan laden.  (Code 31)", NULL },
    { "window.open( scrollbar", NULL },
    { "T68i truc ->", NULL },
    { "T68i ->", NULL },
    { "\"de lijn is bezet\"\"", NULL },
    { "if (eregi(\"", NULL },
    { "This device is not working properly because Windows cannot load the drivers required for this device. (Code 31)", NULL },
    { "execCommand(\"Paste\");", NULL },
    { "\"-1 unread\"", NULL },
    { "\"www.historical-fire-engines", NULL },
    { "\"DVD+RW\" erase", NULL },
    { "[showjekamer)", NULL },
    { "The description for Event ID  1  in Source  True Vector Engine ) cannot be found. The local computer may not have the necessary registry information or message DLL files to display messages from a remote computer. You may be able to use the /AUXSOURC", NULL },
    { "org.apache.jasper.JasperException: This absolute uri (http://java.sun.com/jstl/core) cannot be resolved in either web.xml or the jar files deployed with this application", NULL },
    { "This absolute uri (http://java.sun.com/jstl/core) cannot be resolved in either web.xml or the jar files deployed with this application", NULL },
    { "vervangen # \"/", NULL },
    { "vervangen # /\"", NULL },
    { "while(list($key, $val) = each($HTTP_POST_VARS))", NULL },
    { "PowerDVD does not support the current display mode. (DDraw Overlay mode is recommended)", NULL },
    { "Warning:  Unexpected character in input:  '' (ASCII=92) state=1  highlight", NULL },
    { "error: Qt-1.4 (headers and libraries) not found. Please check your installation!", NULL },
    { "Error while initializing the sound driver: device /dev/dsp can't be opened (No such device) The sound server will continue, using the null output device.", NULL },
    { "mag mijn waarschuwing nu weg ? ;)", NULL },
    { "Abit NF7-S (nForce 2 Chipset) Rev 2.0", NULL },
    { "Setup Could Not Verify the Integrity of the File\" Error Message Occurs When You Try to Install Windows XP Service Pack 1", NULL },
    { "(browser 19) citrix", NULL },
    { "preg_replace (.*?)", NULL },
    { "formule excel #naam\"?\"", NULL },
    { "->", NULL },
    { "De instructie op 0x77f436f7 verwijst naar geheugen op 0x007f4778. De lees-of schrijfbewerking (\"written\") op het geheugen is mislukt", NULL },
    { "<iframe src=\"www.tweakers.net></iframe>", NULL },
    { "\"rpm -e httpd\"", NULL },
    { "automatisch op All Flis (*.*)", NULL },
    { "(Windows; U; Windows NT 5.1; en-US; rv:1.3b) Gecko/20030210", NULL },
    { "en-US; rv:1.3b) Gecko/20030210", NULL },
    { "\"en-US; rv:1.3b) Gecko/20030210\"", NULL },
    { "(./) chmod.sh", NULL },
    { "document.write(ssg(\" html", NULL },
    { "superstack \"mac+adressen\"", NULL },
    { "IIS getenv(REMOTE_HOST)_", NULL },
    { "IIS en getenv(REMOTE_HOST)", NULL },
    { "php getenv(\"HTTP_REFERER\")", NULL },
    { "nec+-1300", NULL },
    { "smbpasswd script \"-s\"", NULL },
    { "leestekens \"  ", NULL },
    { "freesco and (all seeing eye)", NULL },
    { "('all seeing eye') and freesco", NULL },
    { "\"[......\"", NULL },
    { "Error = 11004 (500 No Data (Winsock error #11004))", NULL },
    { "gegevensfout (cyclishe redundantiecontrole)", NULL },
    { "firmware versie waar NEC\"", NULL },
    { "nu.nl \"-1\"", NULL },
    { "provider+-webspace", NULL },
    { "verschil \"dvd+rw\" \"dvd-rw\"", NULL },
    { "(dhcp client) + hangt", NULL },
    { "MSI 875P Neo-FIS2R (Intel 875P)", NULL },
    { "voeding passief gekoeld\"", NULL },
    { "if (mysql_num_rows($resultaat)==1)", NULL },
    { "Server.CreateObject(\"Persits.Upload.1\")", NULL },
    { "if(cod>9999999)cod=parseInt(cod/64)", NULL },
    { "if (cod>9999999", NULL },
    { "\"rm -rf /bin/laden\"", NULL },
    { "\">>> 0) & 0xFF\"", NULL },
    { "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\"> document.body.scrollHeight", NULL },
    { "<BR>window.resizeBy(offsetX,offsetY)<P>kweet", NULL },
    { "linux humor :)", NULL },
    { "ClassFactory kan aangevraagde klasse niet leveren  (Fout=80040111)", NULL },
    { "remote_smtp defer (-44)", NULL },
    { "txtlogin.getText().trim().toUpperCase().intern() == inuser[2 * (i - 1) + 2].trim().toUpperCase().intern() && txtpass.getText().trim().toUpperCase().intern() == inuser[2 * (i - 1) + 3].trim().toUpperCase().intern())", NULL },
    { "Koper + amoniak (NH2", NULL },
    { "nec dvd -/+r", NULL },
    { "er is een gereserveerde fout (-1104) opgetreden", NULL },
    { "Cor \\(CCN\\)'\" <cor.kloet@ccn.controlec.nl>", NULL },
    { "Warning: Failed opening for inclusion (include_path='') in Unknown on line 0", NULL },
    { "\"libstdc++-libc6.2-2.so.3\"", NULL },
    { "\"~\" + \"c:\\\"", NULL },
    { "mysql count(*)", NULL },
    { "for %f in (*.*) do", NULL },
    { "raar \"~\" bestand", NULL },
    { "NEC DVD +-R/RW 1300", NULL },
    { "approved (ref: 38446-263)", NULL },
    { "GA-7VRXP(2.0)", NULL },
    { "~ Could not retrieve directory listing for \"/\"", NULL },
    { "asp CreateObject(\"Word.Document\")", NULL },
    { "De lees- of schrijfbewerking (\"written\") op het geheugen is mislukt.", NULL },
    { "putStr (map (\\x -> chr (round (21/2 * x^3 - 92 * x^2 + 503/2 * x - 105))) [1..4])", NULL },
    { "parent.document.getElementById(\\\"leftmenu\\\").cols", NULL },
    { "<% if not isEmpty(Request.QueryString) then", NULL },
    { "Failed to connect agent. (AGENT=dbaxchg2, EC=UserId =NUll)", NULL },
    { NULL, NULL }
};

int
main(void)
{
    Xapian::QueryParser qp;
    qp.set_stemming_options("english");
    qp.prefixes.insert(pair<string, string>("author", "A"));
    test *p = tests;
    int succeed = 0, fail = 0;
    while (p->query) {
	string expect, parsed;
	if (p->expect)
	    expect = p->expect;
	else
	    expect = "parse error";
	try {
	    string query(p->query);
	    // Need special handling to get a zero byte in here (yuck)
	    if (query == "!!!MAGIC!!!") query = string("\x01weird\x00stuff\x7f", 13);
	    parsed = qp.parse_query(query).get_description();
	    expect = string("Xapian::Query(") + expect + ')';
	} catch (const Xapian::Error &e) {
	    parsed = e.get_msg();
	} catch (const char *s) {
	    parsed = s;
	} catch (...) {
	    parsed = "Unknown exception!";
	}
	if (parsed == expect) {
	    //cout << "OK\t`" << p->query << "'" << endl;
	    if (p->expect) succeed++;
	} else {
	    cout << "Query:\t`" << p->query << "'\n";
	    cout << "Expected:\t`" << expect << "'\n";	    
	    cout << "Got:\t\t`" << parsed << "'\n";
	    fail++;
	}
	p++;
    }
    return (fail != 0);
}
